@@ -1,5 +1,5 @@
 use crate::cache::Cache;
-use crate::{Arc, CompactFile, Data, HashMap, Mutex, SaveOp, Storage};
+use crate::{Arc, CompactFile, Data, HashMap, RwLock, SaveOp, Storage};
 
 /// Inner for SharedPagedData.
 pub struct SPSInner {
@@ -10,7 +10,7 @@ pub struct SPSInner {
 
 /// Allows logical database pages to be shared to allow concurrent readers.
 pub struct SharedPagedData {
-    pub x: Mutex<SPSInner>,
+    pub x: RwLock<SPSInner>,
     pub sp_size: usize,
     pub ep_size: usize,
 }
@@ -22,7 +22,7 @@ impl SharedPagedData {
         let sp_size = file.sp_size;
         let ep_size = file.ep_size;
         Self {
-            x: Mutex::new(SPSInner {
+            x: RwLock::new(SPSInner {
                 file,
                 stash: Cache::new(),
                 cache: HashMap::new(),
@@ -33,7 +33,7 @@ impl SharedPagedData {
     }
     /// Access to a virtual read-only copy of the database logical pages.
     pub fn open_read(self: &Arc<SharedPagedData>) -> AccessPagedData {
-        let mut x = self.x.lock().unwrap();
+        let mut x = self.x.write().unwrap();
         AccessPagedData {
             writer: false,
             time: x.stash.begin_read(),
@@ -51,12 +51,12 @@ impl SharedPagedData {
     }
 
     fn end_read(&self, time: u64) {
-        let mut x = self.x.lock().unwrap();
+        let mut x = self.x.write().unwrap();
         x.stash.end_read(time);
     }
 
     fn set_page(&self, lpnum: u64, p: Data) {
-        let mut x = self.x.lock().unwrap();
+        let mut x = self.x.write().unwrap();
         x.file.set_page(lpnum, &p, p.len());
         let old = {
             if let Some(old) = x.cache.get(&lpnum) {
@@ -70,26 +70,28 @@ impl SharedPagedData {
     }
 
     fn get_page(&self, lpnum: u64, time: u64, writer: bool) -> Data {
-        let mut x = self.x.lock().unwrap();
-        if !writer {
-            if let Some(p) = x.stash.get(lpnum, time) {
+        let p = {
+            let x = self.x.read().unwrap();
+            if !writer {
+                if let Some(p) = x.stash.get(lpnum, time) {
+                    return p.clone();
+                }
+            }
+            if let Some(p) = x.cache.get(&lpnum) {
                 return p.clone();
             }
-        }
-        if let Some(p) = x.cache.get(&lpnum) {
-            p.clone()
-        } else {
             let n = x.file.page_size(lpnum);
             let mut v = vec![0; n];
             x.file.get_page(lpnum, &mut v);
-            let p = Arc::new(v);
-            x.cache.insert(lpnum, p.clone());
-            p
-        }
+            Arc::new(v)
+        };
+        let mut x = self.x.write().unwrap();
+        x.cache.insert(lpnum, p.clone());
+        p
     }
 
     fn free_page(&self, lpnum: u64) {
-        let mut x = self.x.lock().unwrap();
+        let mut x = self.x.write().unwrap();
         x.file.free_page(lpnum);
         x.cache.remove(&lpnum);
     }
@@ -109,7 +111,7 @@ impl AccessPagedData {
     }
     /// Is the underlying file new (so needs to be initialised ).
     pub fn is_new(&self) -> bool {
-        self.writer && self.spd.x.lock().unwrap().file.is_new()
+        self.writer && self.spd.x.read().unwrap().file.is_new()
     }
     /// Check whether compressing a page is worthwhile.
     pub fn compress(&self, size: usize, saving: usize) -> bool {
@@ -124,7 +126,7 @@ impl AccessPagedData {
     /// Allocate a logical page.
     pub fn alloc_page(&self) -> u64 {
         debug_assert!(self.writer);
-        self.spd.x.lock().unwrap().file.alloc_page()
+        self.spd.x.write().unwrap().file.alloc_page()
     }
     /// Free a logical page.
     pub fn free_page(&self, lpnum: u64) {
@@ -134,7 +136,7 @@ impl AccessPagedData {
     /// Commit changes to underlying file ( or rollback logical page allocations ).
     pub fn save(&self, op: SaveOp) {
         debug_assert!(self.writer);
-        let mut x = self.spd.x.lock().unwrap();
+        let mut x = self.spd.x.write().unwrap();
         match op {
             SaveOp::Save => {
                 x.file.save();
